/*
 * main.c
 *
 * Created: 29/09/2012 2:13:52 AM
 *  Author: mdryden
 */ 

#include "asf.h"
#include <stdio.h>
#include <math.h>
#include <avr/io.h>
#include <avr/interrupt.h>

//Internal global variables
volatile bool current_dir = 1;
volatile uint16_t step_counter = 0;
int32_t z_position = 0;

//Internal function declarations
void single_step(void);
uint16_t multi_step(bool dir, uint16_t steps, uint16_t steps_per_s);
//uint16_t mm_step(double mm, double mm_per_s, bool dir);
bool home(void);

void single_step(void){
    static bool last_dir = 1;
    
    if (last_dir != current_dir) {
        ioport_set_pin_level(PIN_DIR, current_dir);
        delay_us(10);
        last_dir = current_dir;
    }
    
    if (current_dir) {
        ++z_position;
    }
    else {
        --z_position;
    }
    
    ioport_set_pin_level(PIN_STEP, 1);
    delay_us(10);
    ioport_set_pin_level(PIN_STEP, 0);
    delay_us(10);
    
    return;
}

uint16_t multi_step(bool dir, uint16_t steps, uint16_t us_per_step){
    current_dir = dir;
    step_counter = 0;
    
    TCCR1A = 0b00000000; // Disable compare
    TCCR1C = 0b00000000; // Disable force output compare
    
    OCR1A = ceil(us_per_step/TC1_TICK_US);
    TCCR1B |= (1 << WGM12); // CTC mode
    
    TIMSK1 |= (1 << OCIE1A); // Enable interrupt
    TCNT1 = 0;
    TCCR1B |= TC1_DIV; // Start Timer
    
    while (step_counter < steps){ // wait until all steps done
        if (ioport_get_pin_level(PIN_ZMIN) == 0) {
            TIMSK1 &= ~(1 << OCIE1A);
            TCCR1B &= ~(0b00000111);
            return step_counter;
        }
    }

    
    // stop timer
    TIMSK1 &= ~(1 << OCIE1A);
    TCCR1B &= ~(0b00000111);
    
    return 0;
}

//uint16_t mm_step(double mm, double mm_per_s, bool dir){
//    multi_step(dir, (uint16_t)mm, (uint16_t)mm_per_s); // mm/steps_per_mm, mm_per_s -> us_per_step
//}
//
//void home(void){
//    while (multi_step(0, 1000, )) {
//        <#statements#>
//    }
//}
bool home(void){
    while (multi_step(0, 1600, 400) == 0);
    z_position = 0;
    return 1;
}


int main(void){
	
	board_init();
	ioport_init();
    cpu_irq_enable(); // enable interrupts
    
	delay_ms(500);
    
    while (1) {
        home();
        delay_s(10);
    }
}

ISR(TIMER1_COMPA_vect){
    single_step();
    step_counter++;
}